<?php

/**
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many individuals
 * and is licensed under the LGPL. For more information please see
 * <http://phing.info>.
 */

namespace Phing\Io;

/**
 * The factory to create fileParsers based on extension name from
 * PhingFile->getFileExtension().
 *
 * @author  Mike Lohmann <mike.lohmann@deck36.de>
 */
class FileParserFactory implements FileParserFactoryInterface
{
    private const XML_FILE_EXTENSION = 'xml';
    /**
     * @const string
     */
    private const YAML_FILE_EXTENSION = 'yml';

    /**
     * @const string
     */
    private const YAML_FILE_EXTENSION_LONG = 'yaml';

    /**
     * {@inheritDoc}
     */
    public function createParser($fileExtension)
    {
        switch ($fileExtension) {
            case self::XML_FILE_EXTENSION:
                $fileParser = new XmlFileParser();

                break;

            case self::YAML_FILE_EXTENSION:
            case self::YAML_FILE_EXTENSION_LONG:
                $fileParser = new YamlFileParser();

                break;

            default:
                $fileParser = new IniFileParser();
        }

        return $fileParser;
    }
}
